//Behaviour.cpp

//This is the implementation file for the behaviour.

#include "Behaviour.h"

//Entities are represented by BMessages which contain all the attributes of a given entity.
//The behaviour algorithm can inspect the environment before deciding what changes to make
//to the attributes.

//BList *prev
//This is a list of pointers to BMessages. Each represents the state of an entity as it
//appeared at the end of the previous generation. This is the "environment".

//BList *next
//This is a list of entities as they should be in the next generation.
//Initially this list is a copy of "prev". The simulator inspects each entity to see what
//behaviours it has. For each behaviour it calls the Behaviour() function of the appropriate
//add-on with the "index" of that entity as a parameter.

//int32 index
//The index of the current entity in the "prev" and "next" lists.
//i.e., current = next->ItemAt(index);

//BList *children
//One possible behaviour is the creation of additional entities. This is done by
//adding the entities (BMessages) to the "children" list.

status_t Behaviour(const BList *prev, const BList *next, int32 index, BList *children)
{
	BMessage *entity = (BMessage *)next->ItemAt(index);

	//Get the details we are interested in
	float x = entity->FindFloat("x-pos");
	float y = entity->FindFloat("y-pos");
	float z = entity->FindFloat("z-pos");

	//Do whatever changes we like
	x = x + 1;
	y = y - 1;
	z = z + 1;

	//Write the details back to the BMessage
	entity->ReplaceFloat("x-pos", x);
	entity->ReplaceFloat("y-pos", y);
	entity->ReplaceFloat("z-pos", z);

	return B_OK;
}

//don't forget to go Window->Settings->x86 Project and change the default file name
