//Behaviour.cpp

//This is the implementation file for the behaviour.

#include "Behaviour.h"

//Entities are represented by BMessages which contain all the attributes of a given entity.
//The behaviour algorithm can inspect the environment before deciding what changes to make
//to the attributes.

//BList *prev
//This is a list of pointers to BMessages. Each represents the state of an entity as it
//appeared at the end of the previous generation. This is the "environment".

//BList *next
//This is a list of entities as they should be in the next generation.
//Initially this list is a copy of "prev". The simulator inspects each entity to see what
//behaviours it has. For each behaviour it calls the Behaviour() function of the appropriate
//add-on with the "index" of that entity as a parameter.

//int32 index
//The index of the current entity in the "prev" and "next" lists.
//i.e., current = next->ItemAt(index);

//BList *children
//One possible behaviour is the creation of additional entities. This is done by
//adding the entities (BMessages) to the "children" list.

status_t Behaviour(const BList *prev, const BList *next, int32 index, BList *children)
{
	BMessage *clone;
	BMessage *current = (BMessage *)prev->ItemAt(index);	
	BMessage *nextgen = (BMessage *)next->ItemAt(index);

	int32 clone_time = current->FindInt32("clone_time");
	int32 trigger =  current->FindInt32("clone_trigger");
	if (trigger < 32)
		trigger = 32;
	
	if (++clone_time == trigger)
	{
		clone = new BMessage(current);

		clone_time = 0;	
		trigger *= 2;	//total numbers should plateau

		if (nextgen->ReplaceInt32("clone_trigger", trigger) < B_OK)
			nextgen->AddInt32("clone_trigger", trigger);

		if (clone->ReplaceInt32("clone_trigger", trigger) < B_OK)
			clone->AddInt32("clone_trigger", trigger);

		clone->ReplaceInt32("clone_time", -trigger);
		clone->RemoveName("initialisers");
		children->AddItem(clone);

	}

	if (nextgen->ReplaceInt32("clone_time", clone_time) < B_OK)
		nextgen->AddInt32("clone_time", clone_time);

	return B_OK;
}

//don't forget to go Window->Settings->x86 Project and change the default file name
