//Behaviour.cpp

//This is the implementation file for the behaviour.

#include "Behaviour.h"

//Entities are represented by BMessages which contain all the attributes of a given entity.
//The behaviour algorithm can inspect the environment before deciding what changes to make
//to the attributes.

//BList *prev
//This is a list of pointers to BMessages. Each represents the state of an entity as it
//appeared at the end of the previous generation. This is the "environment".

//BList *next
//This is a list of entities as they should be in the next generation.
//Initially this list is a copy of "prev". The simulator inspects each entity to see what
//behaviours it has. For each behaviour it calls the Behaviour() function of the appropriate
//add-on with the "index" of that entity as a parameter.

//int32 index
//The index of the current entity in the "prev" and "next" lists.
//i.e., current = next->ItemAt(index);

//BList *children
//One possible behaviour is the creation of additional entities. This is done by
//adding the entities (BMessages) to the "children" list.

status_t Behaviour(const BList *prev, const BList *next, int32 index, BList *children)
{
#if __INTEL__
	const int32 BLUE = 0xFFFF0000;	//ABGR
	const int32 RED = 0xFF0000FF;
	const int32 GREEN = 0xFF00FF00;
#endif

#if __POWERPC__
	const int32 BLUE = 0x0000FFFF;	//RGBA
	const int32 RED = 0xFF0000FF;
	const int32 GREEN = 0x00FF00FF;
#endif
	const int32 WIDTH = 20;

	float x;
	uint32 color;
	status_t err;
	BMessage *entity = (BMessage *)next->ItemAt(index);
	
	//Get the details we are interested in
	if ((err = entity->FindFloat("x-pos", &x)) < B_OK)
	{
		//if it doesn't have the attribute add it
		entity->AddFloat("x-pos", 0);
		x = 0;
	}
	
	if ((err = entity->FindInt32("color", (int32 *)&color)) < B_OK)
	{
		//if it doesn't have the attribute add it
		entity->AddInt32("color", (int32)RED);
		color = RED;
	}
	
	switch(color)
	{
		case RED:
		{
			if (x > 0)
				x--;
			else if (x < 0)
				x++;
			
		}
		break;
		
		case BLUE:
		{
			if (x < WIDTH)
				x++;
			else if (x > WIDTH)
				x--;	
		}
		break;
		
		default:
		{
			if (x < WIDTH / 2)
				x++;
			else if (x > WIDTH / 2)
				x--;	
		}
		break;	
	}

	//Write the details back to the BMessage
	entity->ReplaceFloat("x-pos", x);

	return B_OK;
}

//don't forget to go Window->Settings->x86 Project and change the default file name
