//Behaviour.cpp

//This is the implementation file for the behaviour.

#include "Behaviour.h"
#include <stdlib.h>

//Entities are represented by BMessages which contain all the attributes of a given entity.
//The behaviour algorithm can inspect the environment before deciding what changes to make
//to the attributes.

//BList *prev
//This is a list of pointers to BMessages. Each represents the state of an entity as it
//appeared at the end of the previous generation. This is the "environment".

//BList *next
//This is a list of entities as they should be in the next generation.
//Initially this list is a copy of "prev". The simulator inspects each entity to see what
//behaviours it has. For each behaviour it calls the Behaviour() function of the appropriate
//add-on with the "index" of that entity as a parameter.

//int32 index
//The index of the current entity in the "prev" and "next" lists.
//i.e., current = next->ItemAt(index);

//BList *children
//One possible behaviour is the creation of additional entities. This is done by
//adding the entities (BMessages) to the "children" list.

status_t Behaviour(const BList *prev, const BList *next, int32 index, BList *children)
{
	const int32 ARENA_SIZE = 25;

	BMessage *entity = (BMessage *)next->ItemAt(index);
	
	//Get the attribute we're interested in
	int32 teleport = entity->FindInt32("teleport");

	//See if it is time to teleport
	if (++teleport >= 50)
	{
		teleport = 0;
	
		float x = rand() % ARENA_SIZE;
		float y = rand() % ARENA_SIZE;
		float z = rand() % ARENA_SIZE;

		//Update the entities new position
		if (entity->ReplaceFloat("x-pos", x) < B_OK)
			entity->AddFloat("x-pos", x);

		if (entity->ReplaceFloat("y-pos", y) < B_OK)
			entity->AddFloat("y-pos", y);

		if (entity->ReplaceFloat("z-pos", z) < B_OK)
			entity->AddFloat("z-pos", z);
	}

	//Write back the updated attribute
	if (entity->ReplaceInt32("teleport", teleport) < B_OK)
		entity->AddInt32("teleport", teleport);

	return B_OK;
}

//don't forget to go Window->Settings->x86 Project and change the default file name
