//editorWindow.cpp
//Copyright © 1998, Paul Yu 

#include "editorWindow.h"
#include "message_protocols.h"
#include "listView.h"
#include "navView.h"
#include "controlView.h"
#include "entityWindow.h"

editorWindow::editorWindow() : 
	BWindow(BRect(50,50,100,800), "", B_TITLED_WINDOW, B_NOT_RESIZABLE)
{
	/*	*/
}

editorWindow::~editorWindow()
{
	/*	*/
}

void editorWindow::MessageReceived(BMessage *msg)
{
	switch(msg->what)
	{
		case B_OK:
		{
			BMessenger messenger(mEntityManager);
			mEntities.what = SET_BASELINE;
			messenger.SendMessage(&mEntities);
			Quit();
		}
		break;
		
		case B_CANCEL:
		{
			Quit();
		}
		break;

		case ADD:
		{
			int32 count;
			uint32 type;
		
			BMessage *entity = new BMessage();
			entity->AddBool("alive", true);
			entity->AddString("name", "Unnamed");
			mEntities.AddMessage("entity", entity);
			mListView->AddItem("Unnamed");
			mEntities.GetInfo("entity", &type, &count);
			Editor(entity, count -1);
		}
		break;

		case DUPLICATE:
		{
			BMessage entity;
			int32 index = mListView->GetSelection(NULL);
			
			if ((index >= 0) && (mEntities.FindMessage("entity", index, &entity) == B_OK))
			{
				mEntities.AddMessage("entity", new BMessage(&entity));
				mListView->AddItem(entity.FindString("name"));
			}
		}
		break;

		case UPDATE:
		{
			BMessage entity;
			int32 index;
			
			if ((msg->FindMessage("entity", &entity) == B_OK) &&
				(msg->FindInt32("index", &index) == B_OK))
			{
				mEntities.ReplaceMessage("entity", index, &entity);

				if (entity.HasString("name"))
					mListView->ReplaceItem(entity.FindString("name"), index);
				else
					mListView->ReplaceItem("Unnamed", index);

//Quick 'n' dirty duplication
/*
				mEntities.AddMessage("entity", new BMessage(&entity));
				mEntities.AddMessage("entity", new BMessage(&entity));
				mEntities.AddMessage("entity", new BMessage(&entity));
				mEntities.AddMessage("entity", new BMessage(&entity));
				mEntities.AddMessage("entity", new BMessage(&entity));
				mEntities.AddMessage("entity", new BMessage(&entity));
				mEntities.AddMessage("entity", new BMessage(&entity));
				mEntities.AddMessage("entity", new BMessage(&entity));
				mEntities.AddMessage("entity", new BMessage(&entity));
				mEntities.AddMessage("entity", new BMessage(&entity));
				mListView->AddItem("dupe1");
				mListView->AddItem("dupe2");
				mListView->AddItem("dupe3");
				mListView->AddItem("dupe4");
				mListView->AddItem("dupe5");
				mListView->AddItem("dupe6");
				mListView->AddItem("dupe7");
				mListView->AddItem("dupe8");
				mListView->AddItem("dupe9");
				mListView->AddItem("dupe0");
*/
			}
		}
		break;
		
		case EDIT:
		{
			BMessage entity;
			int32 index = mListView->GetSelection(NULL);
			
			if ((index >= 0) && (mEntities.FindMessage("entity", index, &entity) == B_OK))
				Editor(&entity, index);
		}
		break;

		case DELETE:
		{
			int32 index = mListView->GetSelection(NULL);

			if ((index >= 0) && (mEntities.RemoveData("entity", index) == B_OK))
				mListView->RemoveItem();
		}
		break;

		case SET_CONTEXT:
		{
			BHandler *em;
			BHandler *am;
			BHandler *ee;
			entry_ref ref;
			
			if ((msg->FindPointer("add-on_manager", (void **)&am) == B_OK) &&
				(msg->FindPointer("entity_manager",(void **) &em) == B_OK) &&
				(msg->FindPointer("entity_editor", (void **)&ee) == B_OK) &&
				am && em && ee)
			{
				mAddons = am;
				mEntityManager = em;
				mEntityEditor = ee;
				
				if(msg->FindRef("file", &ref) == B_OK)
				{
					char name[B_FILE_NAME_LENGTH];
					BEntry entry(&ref);
					if (entry.GetName(name) == B_OK)
						SetTitle(name);
					else
						SetTitle("Untitled");
				}
				else
					SetTitle("Untitled");
				
				Show();
			}
			
		}
		break;	
	
		default:
			BWindow::MessageReceived(msg);
	}
}

void editorWindow::Show()
{
	BMessenger messenger(mEntityManager);
	mEntities.what = B_ERROR;
	
	if ((messenger.SendMessage(GET_BASELINE, &mEntities) < B_OK) ||
		(mEntities.what < B_OK))
		return;

	BMessage cancel(B_CANCEL);
	BMessage edit((uint32)B_OK);//EDIT_ENTITY);

	mListView = new listView();
	mListView->SetListTitle("Name");

	BMessage entity;
	int32 index = 0;
	while(mEntities.FindMessage("entity", index++, &entity) == B_OK)
	{
		if (entity.HasString("name"))
			mListView->AddItem(entity.FindString("name"));
		else
			mListView->AddItem("Unnamed");
	}
	AddChild(mListView);

	AddChild(new navView());
	AddChild(new controlView(mListView));

	AddShortcut('d', B_COMMAND_KEY, new BMessage(DUPLICATE));

	BWindow::Show();
}

void editorWindow::Editor(BMessage *entity, int32 index)
{
	BMessage msg(SET_CONTEXT);

	if ((msg.AddPointer("add-on_manager", mAddons) < B_OK) ||
		(msg.AddPointer("editor_window", this) < B_OK) ||
		(msg.AddInt32("index", index) < B_OK) ||
		(msg.AddMessage("entity", entity) < B_OK))
		return;
		
	entityWindow *window = new entityWindow();

	window->Run();
	
	window->PostMessage(&msg);
}
