//entityEditor.cpp
//Copyright © 1998, Paul Yu 

#include <stdio.h>
#include <unistd.h>
#include <StorageKit.h>
#include <Alert.h>
#include "entityEditor.h"
#include "addonManager.h"
#include "editorWindow.h"
#include "message_protocols.h"

entityEditor::entityEditor() : BHandler("entityEditor")
{
	//Get CWD
	app_info ai; 
	BPath path; 
	BEntry entry;

	be_app->GetAppInfo(&ai); 
	entry.SetTo(&ai.ref); 
	entry.GetPath(&path); 
	path.GetParent(&path); 
	get_ref_for_path(path.Path(), &mCurrentDirectory);

	mCurrentFile = NULL;
	
	//Create File Panels
	mOpenPanel = new BFilePanel(B_OPEN_PANEL, NULL, NULL, 0, false);
	mSavePanel = new BFilePanel(B_SAVE_PANEL, NULL, NULL, 0, false);
}

entityEditor::~entityEditor()
{
	fprintf(stderr, "entityEditor::Destructor()\n");

	delete mOpenPanel;
	delete mSavePanel;
	delete mCurrentFile;
}

void entityEditor::MessageReceived(BMessage *msg)
{
	switch(msg->what)
	{
		case SET_CONTEXT:
		{
			addonManager *am;
			BHandler *em;
			BHandler *rm;
			if ((msg->FindPointer("add-on_manager", (void **)&am) == B_OK) &&
				(msg->FindPointer("entity_manager", (void **)&em) == B_OK) &&
				(msg->FindPointer("arena_manager", (void **)&rm) == B_OK) &&
				am && em && rm)
			{
				mAddons = am;
				mEntityManager = em;
				mArenaManager = rm;
			}
		}
		break;

		case EDIT:
		{
			Editor();
		}
		break;

		case NEW:
		{
			BMessage reply;
			BMessage empty(SET_BASELINE);
			BMessenger messenger(mEntityManager);

			messenger.SendMessage(&empty, &reply);
			
			Looper()->PostMessage(RESTART, mArenaManager);

			delete mCurrentFile;
			mCurrentFile = NULL;
		}
		break;

		case OPEN:
		{
			BMessenger messenger(this);
			mOpenPanel->SetTarget(messenger);
			mOpenPanel->SetPanelDirectory(&mCurrentDirectory);
			mOpenPanel->Show();
		}
		break;

		case B_REFS_RECEIVED:
		{
			entry_ref ref;
			mOpenPanel->GetPanelDirectory(&mCurrentDirectory);
			
			if ((msg->FindRef("refs", &ref) < B_OK) ||
				(Open(&ref) < B_OK))
			{
				BAlert *alert =  new BAlert("", "Could not open file.",
					"OK", NULL, NULL, B_WIDTH_AS_USUAL, B_WARNING_ALERT);
				alert->Go();
			}
			else
				Looper()->PostMessage(new BMessage(RESTART), mArenaManager);
		}
		break;

		case SAVE:
		{
			if (mCurrentFile == NULL)
				Looper()->PostMessage(SAVEAS, this);
			else if (Save(mCurrentFile) < B_OK)
			{
				BAlert *alert =  new BAlert("", "Could not write to file.",
					"OK", NULL, NULL, B_WIDTH_AS_USUAL, B_STOP_ALERT);
				alert->Go();
			}
		}
		break;

		case SAVEAS:
		{
			BMessenger messenger(this);
			mSavePanel->SetTarget(messenger);
			mSavePanel->SetPanelDirectory(&mCurrentDirectory);
			mSavePanel->Show();
		}
		break;

		case B_SAVE_REQUESTED:
		{
			status_t err;
			entry_ref dir;
			const char *leaf;
			
			mSavePanel->GetPanelDirectory(&mCurrentDirectory);
			if (((err = msg->FindRef("directory", &dir)) < B_OK) ||
				((err = msg->FindString("name", &leaf)) < B_OK) ||
				((err = SaveAs(&dir, leaf)) < B_OK))
			{
				if (err != B_CANCEL)
				{
					BAlert *alert =  new BAlert("", "Could not write to file.",
						"OK", NULL, NULL, B_WIDTH_AS_USUAL, B_STOP_ALERT);
					alert->Go();
				}
			}
		}
		break;
		
		default:
			BHandler::MessageReceived(msg);
	}
}

void entityEditor::Editor()
{
	BMessage msg(SET_CONTEXT);

	if ((msg.AddPointer("add-on_manager", mAddons) < B_OK) ||
		(msg.AddPointer("entity_manager", mEntityManager) < B_OK) ||
		(msg.AddPointer("entity_editor", this) < B_OK))
		return;
		
	if (mCurrentFile != NULL)
		msg.AddRef("file", mCurrentFile);
		
	editorWindow *window = new editorWindow();	
	window->Run();
	
	window->PostMessage(&msg);
}

status_t entityEditor::Open(entry_ref *ref)
{
	status_t err;
	BFile file(ref, B_READ_ONLY);
	BMessage msg;
	
	if (((err = file.InitCheck()) < B_OK) ||
		((err = msg.Unflatten(&file)) < B_OK))
		return err;
		
	BMessenger messenger(mEntityManager);
	msg.what = SET_BASELINE;
	messenger.SendMessage(&msg);

	delete mCurrentFile;
	mCurrentFile = new entry_ref(*ref);
	
	return B_OK;
}

status_t entityEditor::Save(entry_ref *ref)
{
	status_t err;

	BMessage reply(B_ERROR);
	BMessage msg(GET_BASELINE);
	BMessenger messenger(mEntityManager);
	messenger.SendMessage(&msg, &reply);

	if ((err = reply.what) < B_OK)
		return err;

	{
		BFile file;
		err = file.SetTo(ref, B_WRITE_ONLY | B_CREATE_FILE);
	
		if ((err < B_OK) || 
			((err = reply.Flatten(&file)) < B_OK))
			return err;
	}

	//ref may == mCurrentFile
	entry_ref *new_ref = new entry_ref(*ref);
	delete mCurrentFile;
	mCurrentFile = new_ref;

	return B_OK;
}

status_t entityEditor::SaveAs(entry_ref *dir_ref, const char *leaf)
{
	entry_ref file_ref;
	BDirectory dir(dir_ref);
	BEntry entry(&dir, leaf);
	entry.GetRef(&file_ref);

	return Save(&file_ref);
}
