//lifeWin.cpp

#include <stdio.h>
#include <InterfaceKit.h>
#include <opengl/GLView.h>
#include "lifeWin.h"
#include "entityManager.h"
#include "arenaManager.h"
#include "addonManager.h"
#include "entityEditor.h"
#include "message_protocols.h"

lifeWin::lifeWin()
	: BWindow(BRect(100,100,500,500), "3D Life-forms", B_TITLED_WINDOW, B_NOT_RESIZABLE)
{
	//Create generic BLoopers
	looper1 = new BLooper("Entity Looper", B_LOW_PRIORITY);
	looper2 = new BLooper("Arena Looper", B_LOW_PRIORITY);	

	//Create Managers/Editor (custom BHandlers)
	e_manager = new entityManager();
	looper1->AddHandler(e_manager);
	a_manager = new arenaManager();
	looper2->AddHandler(a_manager);	
	mEditor = new entityEditor();
	looper2->AddHandler(mEditor);
	looper1->Run();
	looper2->Run();

	//Create Addon Manager
	mAddons = new addonManager();

	//Create the MenuBar
	BMenuItem *item;
	BMenuBar *mb = new BMenuBar(BRect(0,0,610,15), "menubar");
	BMenu *menu = new BMenu("©");
	menu->AddItem(new BMenuItem("About", new BMessage(B_ABOUT_REQUESTED)));
	menu->AddItem(new BSeparatorItem());
	menu->AddItem(new BMenuItem("Quit", new BMessage(B_QUIT_REQUESTED), 'q', B_COMMAND_KEY));
	menu->SetTargetForItems(be_app);
	mb->AddItem(menu);
	AddChild(mb);

	//Now we know how high the menubar is...
	float height = (mb->Bounds()).bottom + 1;
	ResizeTo(400, 400 + height);

	//Create a generic BGLView
	BGLView *display = new BGLView(BRect(0,height,400,400 + height), "display", B_FOLLOW_ALL, 0, BGL_RGB | BGL_DEPTH  | BGL_DOUBLE);
	mView = display;
	AddChild(display);

	//Inform the objects of their context
	{
		BMessage msg(SET_CONTEXT);
		msg.AddPointer("arena_manager", a_manager);
		msg.AddPointer("entity_manager", e_manager);
		msg.AddPointer("entity_editor", mEditor);
		msg.AddPointer("client_view", display);
		msg.AddPointer("add-on_manager", mAddons);
		looper1->PostMessage(&msg, e_manager);
		looper2->PostMessage(&msg, mEditor);
		looper2->PostMessage(&msg, a_manager);
	}

	//Display Generation 0
	looper2->PostMessage(DRAW_ARENA, a_manager);

	//Finish creating the menubar
	menu = new BMenu("Baseline");
	menu->AddItem(new BMenuItem("Edit…", new BMessage(EDIT)));
	menu->AddItem(new BSeparatorItem());
	menu->AddItem(new BMenuItem("New", new BMessage(NEW)));
	menu->AddItem(new BMenuItem("Open…", new BMessage(OPEN)));
	menu->AddItem(new BSeparatorItem());
	menu->AddItem(new BMenuItem("Save", new BMessage(SAVE)));
	menu->AddItem(new BMenuItem("Save As…", new BMessage(SAVEAS)));
	menu->SetTargetForItems(mEditor);
	mb->AddItem(menu);
	
	menu = new BMenu("Control");
	menu->AddItem(new BMenuItem("Restart", new BMessage(RESTART), 
					0, B_COMMAND_KEY));
	menu->AddItem(new BMenuItem("Single Step Once", new BMessage(SINGLE_STEP), 
					's', B_COMMAND_KEY));
	menu->AddItem(new BMenuItem("Continuous Run", new BMessage(CONTINUOUS), 
					'r', B_COMMAND_KEY));
	menu->SetTargetForItems(a_manager);
	mb->AddItem(menu);
	
	menu = new BMenu("Rotation");	
	BMenu *xmenu;
	BMenu *ymenu;
	BMenu *zmenu;
	menu->AddItem(new BMenuItem(xmenu = new BMenu("X Rotation"), NULL));
	menu->AddItem(new BMenuItem(ymenu = new BMenu("Y Rotation"), NULL));
	menu->AddItem(new BMenuItem(zmenu = new BMenu("Z Rotation"), NULL));
	
	xmenu->AddItem(item = new BMenuItem("No Spin", new BMessage(X_SPIN_STOP)));
	xmenu->AddItem(new BMenuItem("Spin Faster", new BMessage(X_SPIN_POS),
									'x', B_COMMAND_KEY));
	xmenu->AddItem(new BMenuItem("Spin Slower", new BMessage(X_SPIN_NEG),
									'x', B_CONTROL_KEY));
	
	ymenu->AddItem(item = new BMenuItem("No Spin", new BMessage(Y_SPIN_STOP)));
	ymenu->AddItem(new BMenuItem("Spin Faster", new BMessage(Y_SPIN_POS),
									'y', B_COMMAND_KEY));
	ymenu->AddItem(new BMenuItem("Spin Slower", new BMessage(Y_SPIN_NEG),
									'y', B_CONTROL_KEY));
		
	zmenu->AddItem(item = new BMenuItem("No Spin", new BMessage(Z_SPIN_STOP)));
	zmenu->AddItem(new BMenuItem("Spin Faster", new BMessage(Z_SPIN_POS),
									'z', B_COMMAND_KEY));
	zmenu->AddItem(new BMenuItem("Spin Slower", new BMessage(Z_SPIN_NEG),
									'z', B_CONTROL_KEY));
	xmenu->SetTargetForItems(a_manager);
	ymenu->SetTargetForItems(a_manager);
	zmenu->SetTargetForItems(a_manager);
	mb->AddItem(menu);

	menu = new BMenu("Buffers");
	BMessage message(SET_BUFFER_SIZE);
	message.AddInt32("size", 2);
	item = new BMenuItem("2", new BMessage(&message));
	item->SetMarked(true);
	menu->AddItem(item);
	message.ReplaceInt32("size", 3);
	menu->AddItem(new BMenuItem("3", new BMessage(&message)));
	message.ReplaceInt32("size", 4);
	menu->AddItem(new BMenuItem("4", new BMessage(&message)));
	message.ReplaceInt32("size", 5);
	menu->AddItem(new BMenuItem("5", new BMessage(&message)));
	message.ReplaceInt32("size", 6);
	menu->AddItem(new BMenuItem("6", new BMessage(&message)));
	message.ReplaceInt32("size", 7);
	menu->AddItem(new BMenuItem("7", new BMessage(&message)));
	message.ReplaceInt32("size", 8);
	menu->AddItem(new BMenuItem("8", new BMessage(&message)));
	message.ReplaceInt32("size", 9);
	menu->AddItem(new BMenuItem("9", new BMessage(&message)));
	message.ReplaceInt32("size", 10);
	menu->AddItem(new BMenuItem("10", new BMessage(&message)));
	menu->SetTargetForItems(e_manager);
	menu->SetRadioMode(true);
	mb->AddItem(menu);
}

lifeWin::~lifeWin()
{
	delete mAddons;
}

bool lifeWin::QuitRequested()
{
	//Shutdown
	thread_id tid;
	status_t err;

	//For replicant functionality we should do the right thing
	//and delete the BHandlers explicitly. In applications they
	//they just dissapear when the process dies (the destructor
	//is never called).
	
	//Remove the Entity Manager
	if (looper1->Lock())
	{
		looper1->RemoveHandler(e_manager);
		looper1->Unlock();
		delete e_manager;
	}

	//Tell the Entity Looper to quit (self deleting)
	tid = looper1->Thread();
	looper1->PostMessage(B_QUIT_REQUESTED);
	wait_for_thread(tid, &err);

	//The Arena thread can't finish unless it can lock the window
	//but QuitRequested() automatically locks it in the window thread
	int32 count = 0;
	while (IsLocked())
	{
		Unlock();
		count++;
	};
	fprintf(stderr, "Window locks:%d\n", (int)count);

	//Remove the Arena Manager and the Entity Editor
	if (looper2->Lock())
	{
		looper2->RemoveHandler(a_manager);
		looper2->RemoveHandler(mEditor);
		looper2->Unlock();

		delete a_manager;
		delete mEditor;
	}

	//Tell the Arena Looper to quit	(self deleting)
	tid = looper2->Thread();
	looper2->PostMessage(B_QUIT_REQUESTED);
	wait_for_thread(tid, &err);

	//Replace the locks
	while (count--) Lock();

	//Quit Application
	be_app->PostMessage(B_QUIT_REQUESTED);
	return true;
}
