//orderedList.cpp
//Copyright © 1998, Paul Yu 

#include <stdio.h>
#include "orderedList.h"

static bool Delete(void *object);

namedPointer::namedPointer(const char *name, void *pointer)
{
	mName = new char[strlen(name) + 1];
	strcpy(mName, name);
	mPointer = pointer;
fprintf(stderr, "%s\n", mName);
}

namedPointer::~namedPointer()
{
	delete [] mName;
}

orderedList::orderedList(int32 count) : BList(count)
{
	/*	*/
}

orderedList::~orderedList()
{
	DoForEach(&Delete);
}

status_t orderedList::AddItem(const char *name, void *pointer)
{
	namedPointer *object;

	try
	{
		object = new namedPointer(name, pointer);
	}
	catch(...)
	{
		return B_NO_MEMORY;
	}
	
	bool found;
	int32 index = GetIndex(name, &found);
	
	if (BList::AddItem(object, index))
		return B_OK;

	return B_NO_MEMORY;
}

void *orderedList::FindItem(const char *name)
{
	int32 index;
	bool found;

	index = GetIndex(name, &found);
	if (found)
		return ((namedPointer *)ItemAt(index))->mPointer;
	else
		return NULL;
}

const char *orderedList::GetName(int32 index)
{
	namedPointer *object = (namedPointer *)ItemAt(index);

	if (object)
		return object->mName;
	else
		return NULL;
}

int32 orderedList::GetIndex(const char *name, bool *found)
{
	int32 mid;
	int32 high;
	int32 low =0;
	int32 c;
	namedPointer *cur;
	namedPointer **list;
	
	*found = false;

	if ((high = CountItems() - 1) < 0)
		return 0;

	list = (namedPointer **)Items();
	
	begin:
		mid = (low + high) / 2;
		cur = list[mid];

		c = strcmp(name, cur->mName);

		if (c == 0)
		{
			*found = true;
			return mid;
		}

		if (c < 0)
		{
			high = mid - 1;
			
			if (low > high)
				return mid;
		}
		else
		{
			low = mid + 1;
				
			if (low > high)
				return mid + 1;
		}
		
		goto begin;
}

static bool Delete(void *object)
{
	delete (namedPointer *)object;
	return false;
}
